# $File: //local/member/autrijus/Parse-Binary/t/1-basic.t $ $Author: autrijus $
# $Revision: #3 $ $Change: 1130 $ $DateTime: 2004-02-17T15:40:29.640821Z $

use strict;
use Test;
BEGIN { plan tests => 1 }

use Parse::Binary; ok(1);

1;
